import geopandas
from pyogrio.errors import DataSourceError
from shapely.errors import GEOSException
import zipfile


def is_valid(resource, file):
    """Check if the content is a readable shape file."""

    # There are some strange 'shape files' that are actually ZIP files
    # containing multiple SHP file.
    try:
        with zipfile.ZipFile(file.name, "r") as z:
            shapefiles = [f for f in z.namelist() if f.endswith(".shp")]
    except zipfile.BadZipFile as e:
        resource["error"] = str(e)
        return False

    if len(shapefiles) == 0:
        resource["error"] = "File contains no .shp file"
        return False
    elif (len(shapefiles) == 1) and ("/" not in shapefiles[0]):
        #  the normal case
        with open(file.name, "rb") as f:
            try:
                geopandas.read_file(f)
            except DataSourceError as e:
                resource["error"] = str(e)
                return False
            except GEOSException as e:
                resource["error"] = str(e)
                return False
        return True
    else:
        with zipfile.ZipFile(file.name, "r") as z:
            for shp in shapefiles:
                with z.open(shp) as f:
                    try:
                        geopandas.read_file(f"zip://{file.name}!{shp}")
                    except DataSourceError as e:
                        resource["error"] = str(e)
                        return False
                    except GEOSException as e:
                        resource["error"] = str(e)
                        return False
        return True
